"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsxRenderer = exports.ParseJsx = exports.Fragment = void 0;
var tslib_1 = require("tslib");
var Guid_1 = require("../../System/Guid");
var Strings_1 = require("../../System/Strings");
var EventTypes_1 = require("./EventTypes");
var asap = function (func) {
    setTimeout(function () {
        try {
            func();
        }
        catch (error) {
            console.error(error);
        }
    });
};
exports.Fragment = 'fragment';
function ParseJsx(nodeName, attributes) {
    var children = [];
    for (var _i = 2; _i < arguments.length; _i++) {
        children[_i - 2] = arguments[_i];
    }
    if (typeof nodeName === 'function') {
        return nodeName(attributes ? attributes : {}, children.length ? children : undefined);
    }
    children = [].concat.apply([], tslib_1.__spreadArray([], tslib_1.__read(children), false));
    return { nodeName: nodeName, attributes: attributes, children: children };
}
exports.ParseJsx = ParseJsx;
var JsxRenderer = /** @class */ (function () {
    function JsxRenderer() {
    }
    JsxRenderer.RenderJsx = function (jsx, container) {
        if (container === void 0) { container = document; }
        return JsxRenderer.transformJsxToHtml(jsx, container)
            .outerHTML
            .replace(/<(f|.f)ragment>/g, Strings_1.Strings.Empty);
    };
    JsxRenderer.addElementEventListener = function (attribute, jsx, element, container) {
        var _a;
        var event = attribute.split('on')[1];
        var func = (_a = jsx.attributes) === null || _a === void 0 ? void 0 : _a[attribute];
        var id = (element.attributes['id'] ? element.attributes['id'].nodeValue : Guid_1.Guid.NewGuid());
        element.setAttribute('id', id);
        asap(function () {
            var _a;
            (_a = container.querySelector("[id=\"".concat(id, "\"]"))) === null || _a === void 0 ? void 0 : _a.addEventListener(event, func);
        });
    };
    // eslint-disable-next-line complexity
    JsxRenderer.transformJsxToHtml = function (jsx, container) {
        var e_1, _a;
        var dom = document.createElement(jsx.nodeName);
        for (var key in jsx.attributes) {
            if (EventTypes_1.DomEvents.includes(key)) {
                this.addElementEventListener(key, jsx, dom, container);
            }
            else {
                var value = jsx.attributes[key];
                var shouldAddAttribute = !(typeof value === 'boolean' && value === false);
                if (shouldAddAttribute) {
                    dom.setAttribute(key, jsx.attributes[key].toString());
                }
            }
        }
        try {
            for (var _b = tslib_1.__values(jsx.children || []), _c = _b.next(); !_c.done; _c = _b.next()) {
                var child = _c.value;
                if (typeof child === 'string' || typeof child === 'number') {
                    dom.appendChild(document.createTextNode(child.toString()));
                }
                else if (child) {
                    dom.appendChild(JsxRenderer.transformJsxToHtml(child, container));
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return dom;
    };
    return JsxRenderer;
}());
exports.JsxRenderer = JsxRenderer;
//# sourceMappingURL=Jsx.js.map